"""
Lifeguard core settings
"""
import sys
from os import environ


class SettingsManager(object):
    def __init__(self, settings):
        self.settings = settings

        for entry in self.settings:
            setattr(self, "__{}".format(entry.lower()), self.__get_value(entry))

    def __get_value(self, name):
        options = self.settings[name]
        return environ.get(name, options["default"])

    def read_value(self, name):
        return getattr(self, "__{}".format(name.lower()))


SETTINGS_MANAGER = SettingsManager(
    {
        "LIFEGUARD_SERVER_PORT": {
            "default": "5567",
            "description": "Lifeguard server port number",
        },
        "LIFEGUARD_DIRECTORY": {
            "default": "/data/lifeguard",
            "description": "Location of validations and others resources",
        },
        "LIFEGUARD_LOG_LEVEL": {
            "default": "INFO",
            "description": "Sets the Lifeguard's core log level",
        },
        "LIFEGUARD_NOTIFICATION_IMPLEMENTATIONS": {
            "default": "",
            "description": "Comma-separated list of implementations",
        },
        "LIFEGUARD_VALIDATION_REPOSITORY_IMPLEMENTATION": {
            "default": None,
            "description": "Full package path to validation implementation class",
        },
        "LIFEGUARD_HTTP_PROXY": {
            "default": None,
            "description": "Proxy used to http calls",
        },
        "LIFEGUARD_HTTPS_PROXY": {
            "default": None,
            "description": "Proxy used to https calls",
        },
    }
)

LIFEGUARD_SERVER_PORT = SETTINGS_MANAGER.read_value("LIFEGUARD_SERVER_PORT")
LIFEGUARD_DIRECTORY = SETTINGS_MANAGER.read_value("LIFEGUARD_DIRECTORY")
LOG_LEVEL = SETTINGS_MANAGER.read_value("LIFEGUARD_LOG_LEVEL")
NOTIFICATION_IMPLEMENTATIONS = SETTINGS_MANAGER.read_value(
    "LIFEGUARD_NOTIFICATION_IMPLEMENTATIONS"
)
VALIDATION_REPOSITORY_IMPLEMENTATION = SETTINGS_MANAGER.read_value(
    "LIFEGUARD_VALIDATION_REPOSITORY_IMPLEMENTATION"
)
HTTP_PROXY = SETTINGS_MANAGER.read_value("LIFEGUARD_HTTP_PROXY")
HTTPS_PROXY = SETTINGS_MANAGER.read_value("LIFEGUARD_HTTPS_PROXY")



"""
Lifeguard validation core
"""
from lifeguard.repositories import declare_implementation
from lifeguard.validations import load_validations
from lifeguard.settings import VALIDATION_REPOSITORY_IMPLEMENTATION

VERSION = VERSION = "0.0.5"

NORMAL = "NORMAL"
WARNING = "WARNING"
PROBLEM = "PROBLEM"

ACTION_STATUSES = [NORMAL, WARNING, PROBLEM]


def change_status(old, new):
    if ACTION_STATUSES.index(new) > ACTION_STATUSES.index(old):
        return new
    return old


def setup():
    load_validations()
    declare_implementation("validation", VALIDATION_REPOSITORY_IMPLEMENTATION)



"""
Lifeguard Helpers
"""


def load_implementation(implementation):
    """
    Load and create instance from a string to class
    """
    package_path = implementation.split(".")
    class_name = package_path.pop(-1)
    return getattr(__import__(".".join(package_path)), class_name)()



"""
Interface repositories
"""
from datetime import datetime
from lifeguard.helpers import load_implementation
from lifeguard.logger import lifeguard_logger as logger

IMPLEMENTATIONS = {}


class BaseRepository(object):
    def __init_repository__(self, repository):
        self.__implementation__ = IMPLEMENTATIONS[repository]


class ValidationRepository(BaseRepository):
    def __init__(self):
        BaseRepository.__init_repository__(self, "validation")

    def save_validation_result(self, validation_result):
        validation_result.last_execution = datetime.now()
        self.__implementation__.save_validation_result(validation_result)

    def fetch_last_validation_result(self, validation_name):
        return self.__implementation__.fetch_last_validation_result(validation_name)


def declare_implementation(repository, implementation):

    if not implementation:
        return

    if repository in IMPLEMENTATIONS:
        logger.warning("overwriting implementation for respository %s", repository)
    logger.info(
        "loading implementation %s for repository %s",
        implementation,
        repository,
    )
    IMPLEMENTATIONS[repository] = load_implementation(implementation)



"""
Base of action used to notification
"""
import json

from lifeguard.helpers import load_implementation
from lifeguard.settings import NOTIFICATION_IMPLEMENTATIONS

NOTIFICATION_METHODS = []


def notify_in_thread(validation_response, settings):
    """
    Create a new thread to notify problem status.
    When status returns to NORMAL thread should be closed.

    :param validation_response: a validation response
    :param settings: validation settings
    """

    pass


def notify_in_single_message(validation_response, settings):
    """
    This action sends single message on a new thread.

    To send notification validation response should have:
    {
        "notification": {"notify": True}
    }

    :param validation_response: a validation response
    :param settings: validation settings
    """

    if not NOTIFICATION_METHODS:
        __build_notification_methods()

    response_settings = validation_response.settings or {}
    notification_settings = response_settings.get("notification", {})
    should_notify = notification_settings.get("notify", False)

    if should_notify:
        content = json.dumps(validation_response.details)
        for notification_method in NOTIFICATION_METHODS:
            notification_method.send_single_message(content, settings)


def __build_notification_methods():
    for imp in NOTIFICATION_IMPLEMENTATIONS.split(","):
        NOTIFICATION_METHODS.append(load_implementation(imp))
